using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Globalization;

namespace Seminar
{
    public partial class ControlBinary : UserControl,xmlInterface
    {
        NumberFormatInfo numberInfo;
        public ControlBinary()
        {
            InitializeComponent();
            numberInfo = new NumberFormatInfo();
            numberInfo.CurrencyDecimalSeparator = ".";
        }


        public double UpperBound
        {
            get
            {
                double a;
                if (double.TryParse(textBoxUpperBound.Text,NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign,numberInfo, out a))
                    return a;
                else return double.NaN;
            }
        }

        public double LowerBound
        {
            get
            {
                double a;
                if (double.TryParse(textBoxLowerBound.Text, NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign, numberInfo, out a))
                    return a;
                else return double.NaN;
            }
        }
        public double Precision
        {
            get
            {
                double a;
                if (double.TryParse(textBoxPrecision.Text, out a))
                    return a;
                else return 0;
            }
        }

        public int Dimension
        {
            get
            {
                int a;
                if (Int32.TryParse(textBoxDimension.Text, out a))
                    return a;
                else return 0;
            }
        }
        public bool UseOnePointCrossover
        {
            get
            {
                return checkBoxOnePointCrx.Checked;
            }
        }
        public bool UseUniformCrossover
        {
            get
            {
                return checkBoxUniformCrossover.Checked;
            }
        }

        public bool UseSimpleMutation
        {
            get
            {
                return checkBoxSimpleMutation.Checked;
            }
        }

        public bool UseMixingMutation
        {
            get
            {
                return checkBoxMixingMutation.Checked;
            }
        }
        public bool ToXml(XmlDocument xmlDoc, XmlElement parent)
        {
            XmlElement lBound = xmlDoc.CreateElement("Entry");
            XmlAttribute at = xmlDoc.CreateAttribute("key");
            at.Value = "lbound";
            lBound.SetAttributeNode(at);
            lBound.InnerText = LowerBound.ToString(numberInfo);


            XmlElement uBound = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "ubound";
            uBound.SetAttributeNode(at);
            uBound.InnerText = UpperBound.ToString(numberInfo);


            if (UpperBound == LowerBound || Dimension == 0 || Precision == 0) return false;
            XmlElement precision = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "precision";
            precision.SetAttributeNode(at);
            precision.InnerText = Precision.ToString();

            XmlElement dimension = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "dimension";
            dimension.SetAttributeNode(at);
            dimension.InnerText = Dimension.ToString();
            parent.AppendChild(uBound);
            parent.AppendChild(lBound);
            parent.AppendChild(precision);
            parent.AppendChild(dimension);
            if (checkBoxMixingMutation.Checked)
            {
                XmlElement mixMut = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "mut.mix";
                mixMut.SetAttributeNode(at);
                mixMut.InnerText = "1";
                parent.AppendChild(mixMut);
            }

            if (checkBoxSimpleMutation.Checked)
            {
                XmlElement simpleMut = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "mut.simple";
                simpleMut.SetAttributeNode(at);
                simpleMut.InnerText = "1";
                parent.AppendChild(simpleMut);
            }

            if (checkBoxOnePointCrx.Checked)
            {
                XmlElement onePointCrx = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "crx.onepoint";
                onePointCrx.SetAttributeNode(at);
                onePointCrx.InnerText = "1";
                parent.AppendChild(onePointCrx);
            }

            if (checkBoxOnePointCrx.Checked)
            {
                XmlElement uniformCrx = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "crx.uniform";
                uniformCrx.SetAttributeNode(at);
                uniformCrx.InnerText = "1";
                parent.AppendChild(uniformCrx);
            }

            return true;
        }

        public bool FromXml(XmlElement elem)
        {
            if (elem.Name != "Binary") return false;
            //XmlNodeList entryList   
            foreach (XmlNode node in elem.ChildNodes)
            {
                if (!(node is XmlElement)) continue;
                switch (((XmlElement)node).GetAttribute("key"))
                {
                    case "lbound":
                        textBoxLowerBound.Text = node.InnerText;
                        break;
                    case "ubound":
                        textBoxUpperBound.Text = node.InnerText;
                        break;
                    case "precision":
                        textBoxPrecision.Text = node.InnerText;
                        break;
                    case "dimension":
                        textBoxDimension.Text = node.InnerText;
                        break;
                    case "mut.simple":
                        if (node.InnerText == "1")
                            checkBoxSimpleMutation.Checked = true;
                        break;
                    case "mut.mix":
                        if (node.InnerText == "1")
                            checkBoxMixingMutation.Checked = true;
                        break;
                    case "crx.onepoint":
                        if (node.InnerText == "1")
                            checkBoxOnePointCrx.Checked = true;
                        break;
                    case "crx.uniform":
                        if (node.InnerText == "1")
                            checkBoxUniformCrossover.Checked = true;
                        break;
                }
            }
            return true;
        }

            





            

    }
}
