using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Globalization;

namespace Seminar
{
    public enum genotypeSelect
    {
        random = 0,
        all
    }
    public partial class ControlRegistry : UserControl, xmlInterface
    {
         NumberFormatInfo info;

        int randomizerValue;
        int populationSize;
        double mutIndProb;
        int maxGenerationNumber;
        double fitnessTerminationValue;
        int logLevel;
        string logFile;
        string milestoneFile;
        int milestoneInterval;

        public ControlRegistry()
        {
            InitializeComponent();
            info = new NumberFormatInfo();
            info.CurrencyDecimalSeparator = ".";
            comboBoxMutationSelect.Items.AddRange(new string[] { "random", "all" });
            comboBoxCrossoverSelect.Items.AddRange(new string[] { "random", "all" });
        }
        public int RandomizerValue
        {
            get
            {
                if(Int32.TryParse(textBoxSeed.Text,out randomizerValue))
                    return randomizerValue;
                else return 0;
            }
        }

        public int PopulationSize
        {
            get
            {
                if (Int32.TryParse(textBoxPopSize.Text, out populationSize))
                    return populationSize;
                else return 0;
            }
        }

        public int LogLevel
        {
            get
            {
                return (int)numericUpDownLogLevel.Value;
            }
        }

        public int MilestoneInterval
        {
            get
            {
                if (Int32.TryParse(textBoxMilestoneInterval.Text, out milestoneInterval))
                    return milestoneInterval;
                else return 0;
            }
        }

        public int MaxGenerationNumber
        {
            get
            {
                if (Int32.TryParse(textBoxMaxGenerations.Text, out maxGenerationNumber))
                    return maxGenerationNumber;
                else return 0;
            }
        }

        public double IndividualMutationProbability
        {
            get
            {
                
                if (double.TryParse(textBoxIndMutationProb.Text,NumberStyles.AllowDecimalPoint,info, out mutIndProb))
                    return mutIndProb;
                else return -1;
            }
        }

        public double FitnessTerminationValue
        {
            get
            {
                if (double.TryParse(textBoxFitnessTermination.Text, NumberStyles.AllowDecimalPoint, info, out fitnessTerminationValue))
                    return fitnessTerminationValue;
                else return 0;
            }
        }

        public string LogFilePath
        {
            get
            {
                logFile = textBoxLogFilename.Text;
                return logFile;
            }
        }

        public string MilestoneFilePath
        {
            get
            {
                milestoneFile=textBoxMilestoneFilename.Text;
                return milestoneFile;
            }
        }

        public genotypeSelect MutationGenotypes
        {
            get
            {
                if(comboBoxMutationSelect.SelectedIndex==-1)return genotypeSelect.random;
                else
                {
                    return (genotypeSelect)comboBoxMutationSelect.SelectedIndex;
                }
            }
        }

        public genotypeSelect CrossoverGenotypes
        {
            get
            {
                if(comboBoxCrossoverSelect.SelectedIndex==-1)return genotypeSelect.random;
                else
                {
                    return (genotypeSelect)comboBoxCrossoverSelect.SelectedIndex;
                }
            }
        }



        public bool ToXml(XmlDocument xmlDoc, XmlElement parent)
        {
            
            XmlElement seed = xmlDoc.CreateElement("Entry");
            XmlAttribute at = xmlDoc.CreateAttribute("key");
            at.Value = "randomizer.seed";
            seed.SetAttributeNode(at);
            seed.InnerText = RandomizerValue.ToString();

            if (PopulationSize != 0)
            {
                XmlElement popSize = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "population.size";
                popSize.SetAttributeNode(at);
                popSize.InnerText = PopulationSize.ToString();
                parent.AppendChild(popSize);
            }

            XmlElement mutGenotypes = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "mutation.genotypes";
            mutGenotypes.SetAttributeNode(at);
            mutGenotypes.InnerText = MutationGenotypes.ToString();

            XmlElement crxGenotypes = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "crossover.genotypes";
            crxGenotypes.SetAttributeNode(at);
            crxGenotypes.InnerText = CrossoverGenotypes.ToString();

            parent.AppendChild(seed);
            parent.AppendChild(mutGenotypes);
            parent.AppendChild(crxGenotypes);
            if (LogLevel != 0)
            {
                XmlElement logLv = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "log.level";
                logLv.SetAttributeNode(at);
                logLv.InnerText = LogLevel.ToString();
                parent.AppendChild(logLv);

                XmlElement logFile = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "log.filename";
                logFile.SetAttributeNode(at);
                if (LogFilePath != "")
                    logFile.InnerText = LogFilePath;
                else logFile.InnerText = "log.txt";
                parent.AppendChild(logFile);

            }
            if (IndividualMutationProbability != -1)
            {
                XmlElement mutProb = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "mutation.indprob";
                mutProb.SetAttributeNode(at);
                mutProb.InnerText = mutIndProb.ToString(info);
                parent.AppendChild(mutProb);
            }

            if (MaxGenerationNumber != 0)
            {
                XmlElement maxGen = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "term.maxgen";
                maxGen.SetAttributeNode(at);
                maxGen.InnerText = maxGenerationNumber.ToString();
                parent.AppendChild(maxGen);
            }

            if (FitnessTerminationValue != 0)
            {
                XmlElement termFitness = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "term.fitness";
                termFitness.SetAttributeNode(at);
                termFitness.InnerText = fitnessTerminationValue.ToString(info);
                parent.AppendChild(termFitness);
            }

            if (MilestoneInterval != 0)
            {
                XmlElement mileInt = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "milestone.interval";
                mileInt.SetAttributeNode(at);
                mileInt.InnerText = milestoneInterval.ToString();
                parent.AppendChild(mileInt);

                XmlElement mileFile = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "milestone.filename";
                mileFile.SetAttributeNode(at);
                if (MilestoneFilePath != "")
                    mileFile.InnerText = milestoneFile;
                else mileFile.InnerText = "milestone.txt";
                parent.AppendChild(mileFile);

                
            }
            return true;
        }


        public bool FromXml(XmlElement elem)
        {
            if (elem.Name != "Registry") return false;
            //XmlNodeList entryList   
            foreach (XmlNode node in elem.ChildNodes)
            {
                if (!(node is XmlElement)) continue;
                switch (((XmlElement)node).GetAttribute("key"))
                {
                    case "randomizer.seed":
                        textBoxSeed.Text = node.InnerText;
                        break;
                    case "population.size":
                        textBoxPopSize.Text = node.InnerText;
                        break;
                    case "mutation.indprob":
                        textBoxIndMutationProb.Text = node.InnerText;
                        break;
                    case "term.maxgen":
                        textBoxMaxGenerations.Text = node.InnerText;
                        break;
                    case "term.fitnessval":
                        textBoxFitnessTermination.Text = node.InnerText;
                        break;
                    case "log.level":
                        numericUpDownLogLevel.Value = Int32.Parse(node.InnerText);
                        break;
                    case "log.filename":
                        textBoxLogFilename.Text = node.InnerText;
                        break;
                    case "mutation.genotypes":
                        if (node.InnerText == "random") comboBoxMutationSelect.SelectedIndex = 0;
                        else comboBoxMutationSelect.SelectedIndex = 1;
                        break;
                    case "crossover.genotypes":
                        if (node.InnerText == "random") comboBoxCrossoverSelect.SelectedIndex = 0;
                        else comboBoxCrossoverSelect.SelectedIndex = 1;
                        break;
                    case "milestone.interval":
                        textBoxMilestoneInterval.Text = node.InnerText;
                        break;
                    case "milestone.filename":
                        textBoxMilestoneFilename.Text = node.InnerText;
                        break;
                }
            }
            return true;
        }


        private void button1_Click(object sender, EventArgs e)
        {
            if (saveFileDialogLogMilestone.ShowDialog() != DialogResult.OK) return;
            textBoxLogFilename.Text = saveFileDialogLogMilestone.FileName;
            
            
        }

        private void button2_Click(object sender, EventArgs e)
        {
            saveFileDialogLogMilestone.FileName = "";
            if (saveFileDialogLogMilestone.ShowDialog() != DialogResult.OK) return;
            textBoxMilestoneFilename.Text = saveFileDialogLogMilestone.FileName;
        }
            













            


            

        
     



            
    }
}
