using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace Seminar
{
    public partial class ControlTree : UserControl, xmlInterface
    {
        public ControlTree()
        {
            InitializeComponent();

            comboBoxFunctions.Items.Add("sin");
            comboBoxFunctions.Items.Add("cos");
            comboBoxFunctions.Items.Add("+");
            comboBoxFunctions.Items.Add("-");
            comboBoxFunctions.Items.Add("*");
            comboBoxFunctions.Items.Add("/");
            comboBoxTerminal.Items.Add("x");
        }

        public int MaxDepth
        {
            get
            {
                int a;
                if (Int32.TryParse(textBoxMaxDepth.Text, out a))
                    return a;
                else return 0;
            }
        }

        public int MinDepth
        {
            get
            {
                int a;
                if (Int32.TryParse(textBoxMinDepth.Text, out a))
                    return a;
                else return 0;
            }
        }
        public bool UseSimpleCrossover
        {
            get
            {
                return checkBoxSimpleCrx.Checked;
            }
        }

        public bool UsePermutationMutation
        {
            get
            {
                return checkBoxPermutationMutation.Checked;
            }
        }

        private void buttonAddGenotype_Click(object sender, EventArgs e)
        {
            if(!listBoxFunctionSet.Items.Contains(comboBoxFunctions.SelectedItem))
            listBoxFunctionSet.Items.Add(comboBoxFunctions.SelectedItem);
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            if (listBoxFunctionSet.SelectedIndex == -1) return;
            listBoxFunctionSet.Items.RemoveAt(listBoxFunctionSet.SelectedIndex);
        }

        public List<string> Functions
        {
            get
            {
                List<string> functions = new List<string>();
                foreach (object o in listBoxFunctionSet.Items)
                {
                    functions.Add((string)o);
                }
                return functions;
            }
        }

        public List<string> Terminators
        {
            get
            {
                List<string> terminators = new List<string>();
                foreach (object o in listBoxTerminalSet.Items)
                {
                    terminators.Add((string)o);
                }
                return terminators;
            }
        }

        private void buttonAddTerminal_Click(object sender, EventArgs e)
        {
            if(!listBoxTerminalSet.Items.Contains(comboBoxTerminal.SelectedItem))
            listBoxTerminalSet.Items.Add(comboBoxTerminal.SelectedItem);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (listBoxTerminalSet.SelectedIndex == -1) return;
            listBoxTerminalSet.Items.RemoveAt(listBoxTerminalSet.SelectedIndex);

        }

        public bool ToXml(XmlDocument xmlDoc,XmlElement parent)
        {

            XmlElement maxDepth = xmlDoc.CreateElement("Entry");
            XmlAttribute at = xmlDoc.CreateAttribute("key");
            at.Value = "maxdepth";
            maxDepth.SetAttributeNode(at);
            maxDepth.InnerText = MaxDepth.ToString();


            XmlElement minDepth = xmlDoc.CreateElement("Entry");
            at = xmlDoc.CreateAttribute("key");
            at.Value = "mindepth";
            minDepth.SetAttributeNode(at);
            minDepth.InnerText = MinDepth.ToString();

            if (MaxDepth == 0 
                || MaxDepth < MinDepth 
                ||Functions.Count==0
                || Terminators.Count==0) return false;
            if (Functions.Count != 0)
            {
                XmlElement functionSet = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "functionset";
                functionSet.SetAttributeNode(at);
                functionSet.InnerText = listBoxFunctionSet.Items[0] as string;
                for (int i = 1; i < listBoxFunctionSet.Items.Count; i++)
                {
                    functionSet.InnerText += " "+(listBoxFunctionSet.Items[i] as string);
                }
                parent.AppendChild(functionSet);

            }

            if (listBoxTerminalSet.Items.Count != 0)
            {
                XmlElement terminalSet = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "terminalnset";
                terminalSet.SetAttributeNode(at);
                terminalSet.InnerText = listBoxTerminalSet.Items[0] as string;
                for (int i = 1; i < listBoxTerminalSet.Items.Count; i++)
                {
                    terminalSet.InnerText += " " + (listBoxTerminalSet.Items[i] as string);
                }
                parent.AppendChild(terminalSet);

            }
            if (this.UsePermutationMutation)
            {
                XmlElement perm = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "mut.permutation";
                perm.SetAttributeNode(at);
                perm.InnerText = "1";
                parent.AppendChild(perm);
            }
            if (this.UseSimpleCrossover)
            {
                XmlElement crx = xmlDoc.CreateElement("Entry");
                at = xmlDoc.CreateAttribute("key");
                at.Value = "mut.permutation";
                crx.SetAttributeNode(at);
                crx.InnerText = "1";
                parent.AppendChild(crx);
            }



            //functionSet.InnerText = 
            parent.AppendChild(maxDepth);
            parent.AppendChild(minDepth);

            return true;
        }


        public bool FromXml(XmlElement elem)
        {
            if (elem.Name != "Tree") return false;
            //XmlNodeList entryList   
            foreach (XmlNode node in elem.ChildNodes)
            {
                if (!(node is XmlElement)) continue;
                switch (((XmlElement)node).GetAttribute("key"))
                {
                    case "maxdepth":
                        textBoxMaxDepth.Text = node.InnerText;
                        break;
                    case "mindepth":
                        textBoxMinDepth.Text = node.InnerText;
                        break;
                    case "functionset":
                        string[] funcs = node.InnerText.Split(' ');
                        listBoxFunctionSet.Items.AddRange(funcs);
                        break;
                    case "terminalset":
                        string[]  terms= node.InnerText.Split(' ');
                        listBoxTerminalSet.Items.AddRange(terms);
                        break;
                    case "mut.permutation":
                        if (node.InnerText == "1")
                            checkBoxPermutationMutation.Checked = true;
                        break;
                    case "crx.simple":
                        if (node.InnerText == "1")
                            checkBoxSimpleCrx.Checked = true;
                        break;
                }
            }
            return true;
        }

        




    }
}


