using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Xml;

namespace Seminar
{
    public partial class FormLogViewer : Form
    {
        List<LogMessage> messages;
        List<LogMessage> messagesToDisplay;
        Color[] msgColors;
        public bool Success;
        private ListViewItem convertToListViewItem(LogMessage msg)
        {
            ListViewItem item = new ListViewItem();
            
            item.Text = msg.Level.ToString();
            //item.SubItems.Add();
            item.BackColor = msgColors[msg.Level-1];
            item.SubItems.Add(msg.Message);
            item.ToolTipText = msg.Message;
            //item.SubItems.Add(fileSize);
            return item;
        }
        public FormLogViewer(string logFileName)
        {
            Success = false;
            InitializeComponent();
            FormLogViewer_Load();
            messages=new List<LogMessage>();
            
          /*  messages.Add(new LogMessage(1,"blabla1"));
            messages.Add(new LogMessage(2,"blabla2"));
            messages.Add(new LogMessage(3,"blabla3"));
            messages.Add(new LogMessage(4,"blabla4"));
            messages.Add(new LogMessage(5,"blabla5"));
            messages.Add(new LogMessage(6,"blabla6"));
            messages.Add(new LogMessage(7, "blabla7"));*/
            //logMessageBindingSource.DataSource = messages;
            listView1.Columns.Add("Level");
            listView1.Columns.Add("Message");
            listView1.FullRowSelect = true;
            buttonLv1.Tag = 1;
            buttonLv2.Tag = 2;
            buttonLv3.Tag = 3;
            buttonLv4.Tag = 4;
            buttonLv5.Tag = 5;



            numericUpDownLevel.Value = 5;
            try
            {
                getMessagesFromFile(logFileName);
            }
            catch (Exception e)
            {
                MessageBox.Show("Cannot open log");
                this.Close();
                return;
            }
           
            ShowMessages(5, null);
            ContextMenu ct = new ContextMenu();
            MenuItem menuItem=new MenuItem("Copy");
            menuItem.Click+=new EventHandler(listViewContextMenuClicked);
            ct.MenuItems.Add(menuItem);
            listView1.ContextMenu = ct;
            FormLogViewer_Resize(null, null);
            //bindingSourceMessages.DataSource = messages;
            

        }
        private void ShowMessages(int level,string searchString)
        {
            listView1.Items.Clear();
            foreach(LogMessage logMsg in messages)
            {
                if(logMsg.Level<=level)
                {
                    if((searchString!=null && searchString!="" && logMsg.Message.Contains(searchString))
                        || searchString==null || searchString=="")
                    {
                        listView1.Items.Add(this.convertToListViewItem(logMsg));
                    }
                }
            }
        }
                        
        private void getMessagesFromFile(string filepath)
        {
            TextReader tr=new StreamReader(filepath);
            string line;
            string[] parts;
            bool xmlFormat=false;
            while((line=tr.ReadLine())!=null)
            {
                //parts=line.Split(' ');
                if (line == "") continue;
                if(line.Contains("<log>"))
                {
                    xmlFormat=true;
                    break;
                }

                int level=Int32.Parse(line.Substring(0,line.IndexOf(' ')));
                messages.Add(new LogMessage(level,line.Substring(line.IndexOf(' ')+1)));
            }
            tr.Close();
            if(xmlFormat)
            {
                getMessagesFromXmlFile(filepath);
            }
                
        }
        private void getMessagesFromXmlFile(string filepath)
        {
            XmlDocument xmlDoc=new XmlDocument();
            

            //postoji mogucnost da ce xml file imati visestruke 
            //root cvorove, pa je potrebno koristiti .LoadXml nad preuredjenim
            //dokumentom, izbaciti sve <log> tagove i staviti samo jedan korijen
            TextReader tr = new StreamReader(filepath);
            string fileContent = tr.ReadToEnd();
            fileContent = fileContent.Replace("<log>", "");
            fileContent = fileContent.Replace("</log>", "");
            fileContent = "<log>" + fileContent + "</log>";
            xmlDoc.LoadXml(fileContent);
            XmlNodeList list=xmlDoc.SelectNodes("*/message");
            int level;
            foreach(XmlNode node in list)
            {
                if (!(node is XmlElement)) continue;
                level=Int32.Parse((node as XmlElement).GetAttribute("logLevel"));
                messages.Add(new LogMessage(level,node.InnerText));
            }
            
        }


            


        
        private void FilterChanged(object sender, EventArgs e)
        {
            ShowMessages((int)numericUpDownLevel.Value, textBoxFilter.Text);
        }
        private void listViewContextMenuClicked(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 0) return;
            Clipboard.SetText(listView1.SelectedItems[0].SubItems[1].Text);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            

        }

        private void buttonLv_Click(object sender, EventArgs e)
        {
            if (colorDialog1.ShowDialog() != DialogResult.OK) return;
            else msgColors[(int)(((Control)sender).Tag)-1] = colorDialog1.Color;
            FilterChanged(null, null);
            (sender as Control).BackColor = colorDialog1.Color;
        }

        private void FormLogViewer_Resize(object sender, EventArgs e)
        {
            listView1.Size = new Size(this.Size.Width-40, this.Size.Height- 125 );
            buttonLv1.Location = new Point(buttonLv1.Location.X, this.Size.Height - 70);
            buttonLv2.Location = new Point(buttonLv2.Location.X, this.Size.Height - 70);
            buttonLv3.Location = new Point(buttonLv3.Location.X, this.Size.Height - 70);
            buttonLv4.Location = new Point(buttonLv4.Location.X, this.Size.Height - 70);
            buttonLv5.Location = new Point(buttonLv5.Location.X, this.Size.Height - 70);
            label3.Location = new Point(label3.Location.X, this.Size.Height - 70);
            label4.Location = new Point(label4.Location.X, this.Size.Height - 70);
            label5.Location = new Point(label5.Location.X, this.Size.Height - 70);
            label6.Location = new Point(label6.Location.X, this.Size.Height - 70);
            label7.Location = new Point(label7.Location.X, this.Size.Height - 70);
            //sa column.Width=-2 kazemo da zauzme preostali prostor
            this.listView1.Columns[listView1.Columns.Count - 1].Width = -2;
            
        }

        private void FormLogViewer_FormClosed(object sender, FormClosedEventArgs e)
        {
            
            FileStream fs = new FileStream("Colors", FileMode.Create);
            BinaryWriter bw = new BinaryWriter(fs);
            for (int i = 0; i < msgColors.Length; i++)
                bw.Write(msgColors[i].ToArgb());
            bw.Close();
            fs.Close();
                

        }

        private void FormLogViewer_Load()
        {
            FileStream fs;
            msgColors = new Color[5];
            for (int i = 0; i < msgColors.Length; i++)
            {
                msgColors[i] = Color.Aquamarine;
            }
            try
            {
                fs = new FileStream("Colors", FileMode.Open);
            }
            catch (Exception ex)
            {
                return;
            }
            BinaryReader br=new BinaryReader(fs);
            for (int i = 0; i < msgColors.Length; i++)
                msgColors[i] = Color.FromArgb(br.ReadInt32());
            buttonLv1.BackColor = msgColors[0];
            buttonLv2.BackColor = msgColors[1];
            buttonLv3.BackColor = msgColors[2];
            buttonLv4.BackColor = msgColors[3];
            buttonLv5.BackColor = msgColors[4];
            br.Close();
            fs.Close();

        }







    }
}